class LoopProcessingUnit :
	
	def __init__(self,indent=4) :
		self.indent = indent
		self.loop_bodies = {}

	def FetchLoopBody(self,lines,indent) :
		pass
		
	def ProcessLoop(self,lines) :
		pass
		
	def EvaluateLoop(self,loop_bodies) :
		pass
	